const THREE = require("three");

/**
 * @author maximequiblier
 *
 * @param {Object} params
 * @param {THREE.Vector3} params.up The up vector. For now only support (0,1,0) and (0,0,1)
 * @param {Object} params.materials
 *      @param {THREE.Material} params.materials.front The front material
 *      @param {THREE.Material} params.materials.back The back material
 *      @param {THREE.Material} params.materials.left The left material
 *      @param {THREE.Material} params.materials.right The right material
 *      @param {THREE.Material} params.materials.top The top material
 *      @param {THREE.Material} params.materials.bottom The bottom material
 * @param {Object} params.hover Custom parameters for hover
 *      @param {THREE.Color|Number|String} params.color The material color of the hover element will be replaced with this color on hover
 *      @param {THREE.Color|Number|String} params.outlineColor The outline color. If null, no outline, if undefined, a black outline.
 *
 */
function ViewCubeObject3D(params) {

    THREE.Object3D.call( this );

    var params = params || {}

    this.name = params.name || "";

    this.up = params.up ? params.up : new THREE.Vector3(0,1,0);

    var mat = new THREE.Matrix4();
    if(this.up.z){
        mat.makeRotationAxis(new THREE.Vector3(1,0,0), Math.PI/2)
    }

    var g = new THREE.PlaneBufferGeometry();
    g.translate(0, 0, 0.5);

    var frontg = g.clone();
    frontg.applyMatrix4(mat);
    var backg = g.clone();
    backg.rotateY(Math.PI);
    backg.applyMatrix4(mat);

    var rightg = g.clone();
    rightg.rotateY(Math.PI/2);
    rightg.applyMatrix4(mat);
    var leftg = g.clone();
    leftg.rotateY(-Math.PI/2);
    leftg.applyMatrix4(mat);

    var topg = g.clone();
    topg.rotateX(-Math.PI/2);
    topg.applyMatrix4(mat);
    var botg = g.clone();
    botg.rotateX(Math.PI/2);
    botg.applyMatrix4(mat);

    var frontm = new THREE.Mesh(
        frontg,
        params.materials && params.materials.front ? params.materials.front : new THREE.MeshBasicMaterial({ color: 0x0000ff })
    );
    frontm.name = "front";
    var backm = new THREE.Mesh(
        backg,
        params.materials && params.materials.back ? params.materials.back : new THREE.MeshBasicMaterial({ color: 0x0000ff })
    );
    backm.name = "back";

    var leftm = new THREE.Mesh(
        leftg,
        params.materials && params.materials.left ? params.materials.left : new THREE.MeshBasicMaterial({ color: 0xff0000 })
    );
    leftm.name = "left";
    var rightm = new THREE.Mesh(
        rightg,
        params.materials && params.materials.right ? params.materials.right : new THREE.MeshBasicMaterial({ color: 0xff0000 })
    );
    rightm.name = "right";

    var topm = new THREE.Mesh(
        topg,
        params.materials && params.materials.top ? params.materials.top : new THREE.MeshBasicMaterial({ color: 0x00ff00 })
    );
    topm.name = "top";
    var botm = new THREE.Mesh(
        botg,
        params.materials && params.materials.bottom ? params.materials.bottom : new THREE.MeshBasicMaterial({ color: 0x00ff00 })
    );
    botm.name = "bottom";

    this.add(frontm);
    this.add(backm);
    this.add(rightm);
    this.add(leftm);
    this.add(topm);
    this.add(botm);

    this.hoverMesh = null;
    this.hoverOutline = null;
    if (params.hover && params.hover.outlineColor !== null){
        this.hoverOutline = new THREE.LineSegments(
            null,
            new THREE.LineBasicMaterial({ color: params.hover.outlineColor !== undefined ? params.hover.outlineColor : 0x000000 })
        );
    }

    this.hoverColor = params.hover && params.hover.color ? new THREE.Color(params.hover.color) : new THREE.Color(0x00FFFF);
    this.hoverPrevColor = null;
};

ViewCubeObject3D.prototype = Object.create( THREE.Object3D.prototype );
ViewCubeObject3D.prototype.constructor = ViewCubeObject3D;

ViewCubeObject3D.prototype.getWorldOrientations = (function(){
    var upY = {
        front: new THREE.Euler(0, 0, 0),
        back: new THREE.Euler(Math.PI, 0, -Math.PI),
        left: new THREE.Euler(0, -Math.PI / 2, 0),
        right: new THREE.Euler(0, Math.PI / 2, 0),
        top: new THREE.Euler(-Math.PI / 2, 0, 0),
        bottom: new THREE.Euler(Math.PI / 2, 0, 0)
    };
    var upZ = {
        front: new THREE.Euler(Math.PI/2, 0, 0),
        back: new THREE.Euler(-Math.PI/2, 0, -Math.PI),
        left: new THREE.Euler(Math.PI / 2, -Math.PI / 2, 0),
        right: new THREE.Euler(Math.PI / 2, Math.PI / 2, 0),
        top: new THREE.Euler(0,0,0),
        bottom: new THREE.Euler(-Math.PI, 0, 0)
    };
    return function(){
        if(this.up.z === 1){
            return upZ;
        }else{
            return upY;
        }
    };
})();

ViewCubeObject3D.prototype.hover = function(name){
    if(this.hoverMesh){
        this.hoverMesh.material.color = this.hoverPrevColor;
        this.hoverPrevColor = null;
        this.hoverMesh = null;
        if (this.hoverOutline){
            this.hoverOutline.geometry.dispose();
            this.hoverOutline.parent.remove(this.hoverOutline);
        }
    }
    if(name){
        var o = this.getObjectByName(name);
        if(o){
            this.hoverMesh = o;
            this.hoverPrevColor = this.hoverMesh.material.color;
            this.hoverMesh.material.color = this.hoverColor;
            if (this.hoverOutline){
                this.hoverOutline.geometry = new THREE.EdgesGeometry(this.hoverMesh.geometry);
                this.hoverMesh.add(this.hoverOutline);
            }
        }
    }
};

module.exports = ViewCubeObject3D;